@@ -1,5 +1,6 @@
 package JSON::Validator;
 use Mojo::Base -base;
+
 use Exporter 'import';
 use JSON::Validator::Error;
 use Mojo::File 'path';
@@ -8,9 +9,6 @@ use Mojo::JSON::Pointer;
 use Mojo::URL;
 use Mojo::Util 'deprecated';
 use B;
-use Cwd            ();
-use File::Basename ();
-use File::Spec;
 use Scalar::Util;
 
 use constant VALIDATE_HOSTNAME => eval 'require Data::Validate::Domain;1';
@@ -18,24 +16,26 @@ use constant VALIDATE_IP       => eval 'require Data::Validate::IP;1';
 
 use constant DEBUG => $ENV{JSON_VALIDATOR_DEBUG} || 0;
 
-our $VERSION = '0.92';
+our $VERSION = '0.93';
 our @EXPORT_OK = 'validate_json';
 
+my $BUNDLED_CACHE_DIR = path(path(__FILE__)->dirname, qw(Validator cache));
 my $HTTP_SCHEME_RE = qr{^https?:};
 
 sub E { JSON::Validator::Error->new(@_) }
 sub S { Mojo::Util::md5_sum(Data::Dumper->new([@_])->Sortkeys(1)->Useqq(1)->Dump); }
 
-has cache_dir => sub {
-  deprecated 'cache_dir() is replaced by cache_paths()';
-  shift->cache_paths->[0];
-};
-
 has cache_paths => sub {
   my $self = shift;
-  my @paths = split /:/, ($ENV{JSON_VALIDATOR_CACHE_DIR} || '');
-  push @paths, $self->{cache_dir} if $self->{cache_dir};
-  push @paths, File::Spec->catdir(File::Basename::dirname(__FILE__), qw(Validator cache));
+  my @paths = split /:/, ($ENV{JSON_VALIDATOR_CACHE_PATH} || '');
+
+  if ($ENV{JSON_VALIDATOR_CACHE_DIR}) {
+    warn "JSON_VALIDATOR_CACHE_DIR is deprecated in favor of JSON_VALIDATOR_CACHE_PATH\n"
+      unless $ENV{HARNESS_ACTIVE};
+    push @paths, split /:/, ($ENV{JSON_VALIDATOR_CACHE_DIR} || '');
+  }
+
+  push @paths, $BUNDLED_CACHE_DIR;
   return \@paths;
 };
 
@@ -71,7 +71,7 @@ sub schema {
     $schema = $self->_register_document($schema, $schema->{id});
   }
   else {
-    $schema = Cwd::abs_path($schema) if -e $schema;
+    $schema = path($schema)->to_abs if -e $schema;
     $schema = $self->_load_schema($schema);
   }
 
@@ -126,8 +126,8 @@ sub _load_schema {
   }
   elsif ($parent) {
     $url =~ s!#.*!!;
-    $url = File::Spec->catfile(File::Basename::dirname($parent), split '/', $url);
-    $namespace = Cwd::abs_path($url) || $url;
+    $url = path(path($parent)->dirname, split '/', $url);
+    $namespace = eval { $url->to_abs->to_string } || $url;
   }
 
   # Make sure we create the correct namespace if not already done by Mojo::URL
@@ -158,42 +158,43 @@ sub _load_schema_from_data {
 }
 
 sub _load_schema_from_text {
-  return Mojo::JSON::decode_json($_[1]) if $_[1] =~ /^\s*\{/s;
-  $_[0]->coerce(1);    # need to coerce all values... because YAML is awful :(
+  my $self = shift;
+  return Mojo::JSON::decode_json($_[0]) if $_[0] =~ /^\s*\{/s;
   local $YAML::Syck::ImplicitTyping = 1;
-  _load_yaml($_[1]) || undef;
+  return $self->_load_yaml($_[0]);
 }
 
 sub _load_schema_from_url {
   my ($self, $url, $namespace) = @_;
-  my $cache_dir  = $self->cache_paths->[0];
+  my $cache_path = $self->cache_paths->[0];
   my $cache_file = Mojo::Util::md5_sum($namespace);
   my $tx;
 
   for (@{$self->cache_paths}) {
-    my $path = File::Spec->catfile($_, $cache_file);
+    my $path = path $_, $cache_file;
     next unless -r $path;
     warn "[JSON::Validator] Loading cached file $path\n" if DEBUG;
-    return path($path)->slurp;
+    return $path->slurp;
   }
 
   $tx = $self->ua->get($url);
   die $tx->error->{message} if $tx->error;
 
-  if ($cache_dir and -w $cache_dir) {
-    $cache_file = File::Spec->catfile($cache_dir, $cache_file);
-    Mojo::Util::spurt($tx->res->body, $cache_file);
+  if ($cache_path and $cache_path ne $BUNDLED_CACHE_DIR and -w $cache_path) {
+    $cache_file = path $cache_path, $cache_file;
+    warn "[JSON::Validator] Caching $namespace to $cache_file\n" unless $ENV{HARNESS_ACTIVE};
+    $cache_file->spurt($tx->res->body);
   }
 
   return $tx->res->body;
 }
 
 sub _default_id {
-  my $path = Cwd::abs_path($0);
+  my $path = path($0)->to_abs;
   state $id = 0;
-  $path = File::Basename::dirname($path) if $path;
-  $path = Cwd::getcwd unless $path;
-  return File::Spec->catfile($path, sprintf 'json-validator-%s.json', ++$id);
+  $path = $path->dirname if $path;
+  $path = path() unless $path;
+  return path($path, sprintf 'json-validator-%s.json', ++$id)->to_string;
 }
 
 sub _register_document {
@@ -746,14 +747,28 @@ sub _is_uri { $_[0] =~ qr!^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*
 # Please report if you need to manually monkey patch this function
 # https://github.com/jhthorsen/json-validator/issues
 sub _load_yaml {
-  require List::Util;
-  my @YAML_MODULES = qw(YAML::XS YAML::Syck);    # subject to change
-  my $YAML_MODULE = (List::Util::first { eval "require $_;1" } @YAML_MODULES)[0];
-  die "Need to install one of these YAML modules: @YAML_MODULES (YAML::XS is recommended)"
-    unless $YAML_MODULE;
-  warn "[JSON::Validator] Using $YAML_MODULE to parse YAML\n" if DEBUG;
-  Mojo::Util::monkey_patch(__PACKAGE__, _load_yaml => eval "\\\&$YAML_MODULE\::Load");
-  _load_yaml(@_);
+  my $self = shift->coerce(1);
+  my $visit;
+
+  state $yaml_module = do {
+    require List::Util;
+    my @modules = qw(YAML::XS YAML::Syck);                                  # subject to change
+    my $module = (List::Util::first { eval "require $_;1" } @modules)[0];
+    die "Need to install one of these YAML modules: @modules (YAML::XS is recommended)"
+      unless $module;
+    warn "[JSON::Validator] Using $module to parse YAML\n" if DEBUG;
+    $module;
+  };
+
+  $visit = sub {
+    my $v = shift;
+    $visit->($_) for grep { ref $_ eq 'HASH' } values %$v;
+    return $v unless $v->{type} and $v->{type} eq 'boolean' and exists $v->{default};
+    %$v = (%$v, default => $v->{default} ? Mojo::JSON->true : Mojo::JSON->false);
+    return $v;
+  };
+
+  return $visit->($yaml_module->can('Load')->($_[0]));
 }
 
 sub _merge_errors {
@@ -785,7 +800,7 @@ JSON::Validator - Validate data against a JSON schema
 
 =head1 VERSION
 
-0.92
+0.93
 
 =head1 SYNOPSIS
 
@@ -855,8 +870,16 @@ This module comes with some JSON specifications bundled, so your application
 don't have to fetch those from the web. These specifications should be up to
 date, but please submit an issue if they are not.
 
-Note that the bundled specifications can be ignored, by customizing
-L</cache_paths>.
+Files referenced to an URL will automatically be cached if the first element in
+L</cache_paths> is a writable directory. Note that the cache headers for the
+remote assets are B<not> honored, so you will manually need to remove any
+cached file, should you need to refresh them.
+
+To download and cache an online asset, do this:
+
+  JSON_VALIDATOR_CACHE_PATH=/some/writable/directory perl myapp.pl
+
+Here is the list of the bundled specifications:
 
 =over 2
 
@@ -961,27 +984,18 @@ See also L</validate> and L</ERROR OBJECT> for more details.
 
 =head1 ATTRIBUTES
 
-=head2 cache_dir
-
-Deprecated in favor of L</cache_paths>.
-
 =head2 cache_paths
 
   $self = $self->cache_paths(\@paths);
   $array_ref = $self->cache_paths;
 
-Search paths to where cached specifications are stored. Defaults to
-C<JSON_VALIDATOR_CACHE_DIR> and the bundled spec files that are shipped with
-this distribution.
+A list of directories to where cached specifications are stored. Defaults to
+C<JSON_VALIDATOR_CACHE_PATH> environment variable and the specs that is bundled
+with this distribution.
 
-  JSON_VALIDATOR_CACHE_DIR=/cache/dir:/some/other/location perl script.pl
+C<JSON_VALIDATOR_CACHE_PATH> can be a list of directories, each separated by ":".
 
-To download a file and add it to the cache, do this:
-
-  $ curl http://swagger.io/v2/schema.json > /cache/dir/$(md5 -qs http://swagger.io/v2/schema.json)
-
-Files referenced to an URL will automatically be cached if the first path in
-L</cache_paths> is writable.
+See L</Bundled specifications> for more details.
 
 =head2 formats
 
